@@ -18,11 +18,18 @@
 #ifndef GROONGA_H
 #define GROONGA_H
 
+#include "groonga/portability.h"
 #include "groonga/groonga.h"
+#include "groonga/conf.h"
+#include "groonga/obj.h"
 #include "groonga/ii.h"
+#include "groonga/geo.h"
 #include "groonga/expr.h"
 #include "groonga/output.h"
 #include "groonga/util.h"
 #include "groonga/request_canceler.h"
+#include "groonga/thread.h"
+#include "groonga/windows.h"
+#include "groonga/windows_event_logger.h"
 
 #endif /* GROONGA_H */
