@@ -18,12 +18,14 @@
 #ifndef GROONGA_GROONGA_H
 #define GROONGA_GROONGA_H
 
+#include <stdarg.h>
+#include <sys/types.h>
+#include <stdint.h>
+
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
-#include <stdarg.h>
-
 #ifndef GRN_API
 # if defined(_WIN32) || defined(_WIN64)
 #  define GRN_API __declspec(dllimport)
@@ -121,6 +123,7 @@ typedef enum {
   GRN_TOKEN_FILTER_ERROR = -73,
   GRN_COMMAND_ERROR = -74,
   GRN_PLUGIN_ERROR = -75,
+  GRN_SCORER_ERROR = -76
 } grn_rc;
 
 GRN_API grn_rc grn_init(void);
@@ -159,6 +162,9 @@ typedef enum {
   GRN_LOG_DUMP
 } grn_log_level;
 
+GRN_API const char *grn_log_level_to_string(grn_log_level level);
+GRN_API grn_bool grn_log_level_parse(const char *string, grn_log_level *level);
+
 /* query log flags */
 #define GRN_QUERY_LOG_NONE        (0x00)
 #define GRN_QUERY_LOG_COMMAND     (0x01<<0)
@@ -181,7 +187,8 @@ typedef enum {
   GRN_CONTENT_TSV,
   GRN_CONTENT_JSON,
   GRN_CONTENT_XML,
-  GRN_CONTENT_MSGPACK
+  GRN_CONTENT_MSGPACK,
+  GRN_CONTENT_GROONGA_COMMAND_LIST
 } grn_content_type;
 
 typedef struct _grn_obj grn_obj;
@@ -399,6 +406,7 @@ struct _grn_obj {
 
 #define GRN_OBJ_REFER                  (0x01<<0)
 #define GRN_OBJ_OUTPLACE               (0x01<<1)
+#define GRN_OBJ_OWN                    (0x01<<5)
 
 #define GRN_OBJ_INIT(obj,obj_type,obj_flags,obj_domain) do { \
   (obj)->header.type = (obj_type);\
@@ -427,10 +435,19 @@ GRN_API grn_obj *grn_db_create(grn_ctx *ctx, const char *path, grn_db_create_opt
 GRN_API grn_obj *grn_db_open(grn_ctx *ctx, const char *path);
 GRN_API void grn_db_touch(grn_ctx *ctx, grn_obj *db);
 GRN_API grn_rc grn_db_recover(grn_ctx *ctx, grn_obj *db);
+GRN_API grn_rc grn_db_unmap(grn_ctx *ctx, grn_obj *db);
 
 GRN_API grn_rc grn_ctx_use(grn_ctx *ctx, grn_obj *db);
 GRN_API grn_obj *grn_ctx_db(grn_ctx *ctx);
 GRN_API grn_obj *grn_ctx_get(grn_ctx *ctx, const char *name, int name_size);
+GRN_API grn_rc grn_ctx_get_all_tables(grn_ctx *ctx, grn_obj *tables_buffer);
+GRN_API grn_rc grn_ctx_get_all_types(grn_ctx *ctx, grn_obj *types_buffer);
+GRN_API grn_rc grn_ctx_get_all_tokenizers(grn_ctx *ctx,
+                                          grn_obj *tokenizers_buffer);
+GRN_API grn_rc grn_ctx_get_all_normalizers(grn_ctx *ctx,
+                                           grn_obj *normalizers_buffer);
+GRN_API grn_rc grn_ctx_get_all_token_filters(grn_ctx *ctx,
+                                             grn_obj *token_filters_buffer);
 
 typedef enum {
   GRN_DB_VOID = 0,
@@ -463,14 +480,18 @@ typedef enum {
 } grn_builtin_tokenizer;
 
 GRN_API grn_obj *grn_ctx_at(grn_ctx *ctx, grn_id id);
+GRN_API grn_bool grn_ctx_is_opened(grn_ctx *ctx, grn_id id);
 
 GRN_API grn_obj *grn_type_create(grn_ctx *ctx, const char *name, unsigned int name_size,
                                  grn_obj_flags flags, unsigned int size);
 
 GRN_API grn_rc grn_plugin_register(grn_ctx *ctx, const char *name);
+GRN_API grn_rc grn_plugin_unregister(grn_ctx *ctx, const char *name);
 GRN_API grn_rc grn_plugin_register_by_path(grn_ctx *ctx, const char *path);
+GRN_API grn_rc grn_plugin_unregister_by_path(grn_ctx *ctx, const char *path);
 GRN_API const char *grn_plugin_get_system_plugins_dir(void);
 GRN_API const char *grn_plugin_get_suffix(void);
+GRN_API const char *grn_plugin_get_ruby_suffix(void);
 
 typedef struct {
   const char *name;
@@ -487,7 +508,8 @@ typedef enum {
   GRN_PROC_FUNCTION,
   GRN_PROC_HOOK,
   GRN_PROC_NORMALIZER,
-  GRN_PROC_TOKEN_FILTER
+  GRN_PROC_TOKEN_FILTER,
+  GRN_PROC_SCORER
 } grn_proc_type;
 
 GRN_API grn_obj *grn_proc_create(grn_ctx *ctx,
@@ -562,11 +584,11 @@ GRN_API grn_obj *grn_table_cursor_table(grn_ctx *ctx, grn_table_cursor *tc);
 
 typedef struct {
   grn_id rid;
-  grn_id sid;
-  unsigned int pos;
-  unsigned int tf;
-  unsigned int weight;
-  unsigned int rest;
+  uint32_t sid;
+  uint32_t pos;
+  uint32_t tf;
+  uint32_t weight;
+  uint32_t rest;
 } grn_posting;
 
 GRN_API grn_obj *grn_index_cursor_open(grn_ctx *ctx, grn_table_cursor *tc, grn_obj *index,
@@ -707,10 +729,26 @@ typedef enum {
   GRN_OP_TABLE_SORT,
   GRN_OP_TABLE_GROUP,
   GRN_OP_JSON_PUT,
-  GRN_OP_GET_MEMBER
+  GRN_OP_GET_MEMBER,
+  GRN_OP_REGEXP
 } grn_operator;
 
 GRN_API const char *grn_operator_to_string(grn_operator op);
+GRN_API grn_bool grn_operator_exec_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_not_equal(grn_ctx *ctx,
+                                             grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_less(grn_ctx *ctx, grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_greater(grn_ctx *ctx, grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_less_equal(grn_ctx *ctx,
+                                              grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_greater_equal(grn_ctx *ctx,
+                                                 grn_obj *x, grn_obj *y);
+GRN_API grn_bool grn_operator_exec_match(grn_ctx *ctx,
+                                         grn_obj *target, grn_obj *sub_text);
+GRN_API grn_bool grn_operator_exec_prefix(grn_ctx *ctx,
+                                          grn_obj *target, grn_obj *prefix);
+GRN_API grn_bool grn_operator_exec_regexp(grn_ctx *ctx,
+                                          grn_obj *target, grn_obj *pattern);
 
 struct _grn_table_group_result {
   grn_obj *table;
@@ -825,8 +863,6 @@ GRN_API grn_obj *grn_obj_get_element_info(grn_ctx *ctx, grn_obj *obj, grn_id id,
 GRN_API grn_rc grn_obj_set_element_info(grn_ctx *ctx, grn_obj *obj, grn_id id,
                                         grn_info_type type, grn_obj *value);
 
-GRN_API grn_bool grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj);
-
 GRN_API grn_obj *grn_obj_get_value(grn_ctx *ctx, grn_obj *obj, grn_id id, grn_obj *value);
 GRN_API int grn_obj_get_values(grn_ctx *ctx, grn_obj *obj, grn_id offset, void **values);
 
@@ -885,6 +921,8 @@ GRN_API grn_rc grn_obj_lock(grn_ctx *ctx, grn_obj *obj, grn_id id, int timeout);
 GRN_API grn_rc grn_obj_unlock(grn_ctx *ctx, grn_obj *obj, grn_id id);
 GRN_API grn_rc grn_obj_clear_lock(grn_ctx *ctx, grn_obj *obj);
 GRN_API unsigned int grn_obj_is_locked(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_flush(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_flush_recursive(grn_ctx *ctx, grn_obj *obj);
 GRN_API int grn_obj_defrag(grn_ctx *ctx, grn_obj *obj, int threshold);
 
 GRN_API grn_obj *grn_obj_db(grn_ctx *ctx, grn_obj *obj);
@@ -901,6 +939,9 @@ struct _grn_search_optarg {
   int vector_size;
   grn_obj *proc;
   int max_size;
+  grn_obj *scorer;
+  grn_obj *scorer_args_expr;
+  unsigned int scorer_args_expr_offset;
 };
 
 GRN_API grn_rc grn_obj_search(grn_ctx *ctx, grn_obj *obj, grn_obj *query,
@@ -926,6 +967,10 @@ GRN_API grn_rc grn_vector_add_element(grn_ctx *ctx, grn_obj *vector,
 GRN_API unsigned int grn_vector_get_element(grn_ctx *ctx, grn_obj *vector,
                                             unsigned int offset, const char **str,
                                             unsigned int *weight, grn_id *domain);
+GRN_API unsigned int grn_vector_pop_element(grn_ctx *ctx, grn_obj *vector,
+                                            const char **str,
+                                            unsigned int *weight,
+                                            grn_id *domain);
 
 /*-------------------------------------------------------------
  * grn_uvector
@@ -966,9 +1011,22 @@ GRN_API grn_rc grn_obj_delete_hook(grn_ctx *ctx, grn_obj *obj, grn_hook_entry en
 
 GRN_API grn_obj *grn_obj_open(grn_ctx *ctx, unsigned char type, grn_obj_flags flags, grn_id domain);
 
+/* Deprecated since 5.0.1. Use grn_column_find_index_data() instead. */
 GRN_API int grn_column_index(grn_ctx *ctx, grn_obj *column, grn_operator op,
                              grn_obj **indexbuf, int buf_size, int *section);
 
+/* @since 5.0.1. */
+typedef struct _grn_index_datum {
+  grn_obj *index;
+  unsigned int section;
+} grn_index_datum;
+
+/* @since 5.0.1. */
+GRN_API unsigned int grn_column_find_index_data(grn_ctx *ctx, grn_obj *column,
+                                                grn_operator op,
+                                                grn_index_datum *index_data,
+                                                unsigned int n_index_data);
+
 GRN_API grn_rc grn_obj_delete_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, grn_bool removep);
 GRN_API grn_rc grn_obj_path_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, char *buffer);
 
@@ -1117,10 +1175,10 @@ GRN_API void grn_logger_set_max_level(grn_ctx *ctx, grn_log_level max_level);
 GRN_API grn_log_level grn_logger_get_max_level(grn_ctx *ctx);
 
 #ifdef __GNUC__
-#define GRN_ATTRIBUTE_PRINTF(fmt_pos) \
+# define GRN_ATTRIBUTE_PRINTF(fmt_pos) \
   __attribute__ ((format(printf, fmt_pos, fmt_pos + 1)))
 #else
-#define GRN_ATTRIBUTE_PRINTF(fmt_pos)
+# define GRN_ATTRIBUTE_PRINTF(fmt_pos)
 #endif /* __GNUC__ */
 
 GRN_API void grn_logger_put(grn_ctx *ctx, grn_log_level level,
@@ -1130,13 +1188,15 @@ GRN_API void grn_logger_reopen(grn_ctx *ctx);
 GRN_API grn_bool grn_logger_pass(grn_ctx *ctx, grn_log_level level);
 
 #ifndef GRN_LOG_DEFAULT_LEVEL
-#define GRN_LOG_DEFAULT_LEVEL GRN_LOG_NOTICE
+# define GRN_LOG_DEFAULT_LEVEL GRN_LOG_NOTICE
 #endif /* GRN_LOG_DEFAULT_LEVEL */
 
 GRN_API void grn_default_logger_set_max_level(grn_log_level level);
 GRN_API grn_log_level grn_default_logger_get_max_level(void);
 GRN_API void grn_default_logger_set_path(const char *path);
 GRN_API const char *grn_default_logger_get_path(void);
+GRN_API void grn_default_logger_set_rotate_threshold_size(off_t threshold);
+GRN_API off_t grn_default_logger_get_rotate_threshold_size(void);
 
 #define GRN_LOG(ctx,level,...) do {\
   if (grn_logger_pass(ctx, level)) {\
@@ -1169,6 +1229,8 @@ GRN_API void grn_default_query_logger_set_flags(unsigned int flags);
 GRN_API unsigned int grn_default_query_logger_get_flags(void);
 GRN_API void grn_default_query_logger_set_path(const char *path);
 GRN_API const char *grn_default_query_logger_get_path(void);
+GRN_API void grn_default_query_logger_set_rotate_threshold_size(off_t threshold);
+GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
 
 #define GRN_QUERY_LOG(ctx, flag, mark, format, ...) do {\
   if (grn_query_logger_pass(ctx, flag)) {\
@@ -1265,82 +1327,10 @@ GRN_API grn_rc grn_text_printf(grn_ctx *ctx, grn_obj *bulk,
 GRN_API grn_rc grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk,
                                 const char *format, va_list args);
 
-typedef struct _grn_obj_format grn_obj_format;
-
-#define GRN_OBJ_FORMAT_WITH_COLUMN_NAMES   (0x01<<0)
-#define GRN_OBJ_FORMAT_AS_ARRAY            (0x01<<3)
-/* Deprecated since 4.0.1. It will be removed at 5.0.0.
-   Use GRN_OBJ_FORMAT_AS_ARRAY instead.*/
-#define GRN_OBJ_FORMAT_ASARRAY             GRN_OBJ_FORMAT_AS_ARRAY
-#define GRN_OBJ_FORMAT_WITH_WEIGHT         (0x01<<4)
-
-struct _grn_obj_format {
-  grn_obj columns;
-  const void *min;
-  const void *max;
-  unsigned int min_size;
-  unsigned int max_size;
-  int nhits;
-  int offset;
-  int limit;
-  int hits_offset;
-  int flags;
-  grn_obj *expression;
-};
-
-#define GRN_OBJ_FORMAT_INIT(format,format_nhits,format_offset,format_limit,format_hits_offset) do { \
-  GRN_PTR_INIT(&(format)->columns, GRN_OBJ_VECTOR, GRN_ID_NIL);\
-  (format)->nhits = (format_nhits);\
-  (format)->offset = (format_offset);\
-  (format)->limit = (format_limit);\
-  (format)->hits_offset = (format_hits_offset);\
-  (format)->flags = 0;\
-  (format)->expression = NULL;\
-} while (0)
-
-#define GRN_OBJ_FORMAT_FIN(ctx,format) do {\
-  int ncolumns = GRN_BULK_VSIZE(&(format)->columns) / sizeof(grn_obj *);\
-  grn_obj **columns = (grn_obj **)GRN_BULK_HEAD(&(format)->columns);\
-  while (ncolumns--) { grn_obj_unlink((ctx), *columns++); }\
-  GRN_OBJ_FIN((ctx), &(format)->columns);\
-  if ((format)->expression) { GRN_OBJ_FIN((ctx), (format)->expression); } \
-} while (0)
-
-GRN_API void grn_output_obj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
-                            grn_obj *obj, grn_obj_format *format);
-GRN_API void grn_output_envelope(grn_ctx *ctx, grn_rc rc,
-                                 grn_obj *head, grn_obj *body, grn_obj *foot,
-                                 const char *file, int line);
-
-GRN_API void grn_ctx_output_flush(grn_ctx *ctx, int flags);
-GRN_API void grn_ctx_output_array_open(grn_ctx *ctx,
-                                       const char *name, int nelements);
-GRN_API void grn_ctx_output_array_close(grn_ctx *ctx);
-GRN_API void grn_ctx_output_map_open(grn_ctx *ctx,
-                                     const char *name, int nelements);
-GRN_API void grn_ctx_output_map_close(grn_ctx *ctx);
-GRN_API void grn_ctx_output_int32(grn_ctx *ctx, int value);
-GRN_API void grn_ctx_output_int64(grn_ctx *ctx, long long int value);
-GRN_API void grn_ctx_output_float(grn_ctx *ctx, double value);
-GRN_API void grn_ctx_output_cstr(grn_ctx *ctx, const char *value);
-GRN_API void grn_ctx_output_str(grn_ctx *ctx,
-                                const char *value, unsigned int value_len);
-GRN_API void grn_ctx_output_bool(grn_ctx *ctx, grn_bool value);
-GRN_API void grn_ctx_output_obj(grn_ctx *ctx,
-                                grn_obj *value, grn_obj_format *format);
-
-
-GRN_API grn_content_type grn_ctx_get_output_type(grn_ctx *ctx);
-GRN_API grn_rc grn_ctx_set_output_type(grn_ctx *ctx, grn_content_type type);
-GRN_API const char *grn_ctx_get_mime_type(grn_ctx *ctx);
 GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
                                       void (*func)(grn_ctx *, int, void *),
                                       void *func_arg);
 
-/* obsolete */
-GRN_API grn_rc grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj,
-                             grn_obj_format *format);
-
 /* various values exchanged via grn_obj */
 
 #define GRN_OBJ_DO_SHALLOW_COPY        (GRN_OBJ_REFER|GRN_OBJ_OUTPLACE)
@@ -1409,7 +1399,8 @@ GRN_API grn_rc grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj,
 #define GRN_RECORD_INIT GRN_VALUE_FIX_SIZE_INIT
 #define GRN_PTR_INIT(obj,flags,domain)\
   GRN_OBJ_INIT((obj), ((flags) & GRN_OBJ_VECTOR) ? GRN_PVECTOR : GRN_PTR,\
-               ((flags) & GRN_OBJ_DO_SHALLOW_COPY), (domain))
+               ((flags) & (GRN_OBJ_DO_SHALLOW_COPY | GRN_OBJ_OWN)),\
+               (domain))
 #define GRN_TOKYO_GEO_POINT_INIT(obj,flags) \
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_TOKYO_GEO_POINT)
 #define GRN_WGS84_GEO_POINT_INIT(obj,flags) \
