@@ -8,7 +8,7 @@ class ScriptLoader
   end
 
   def load_once
-    if absolete_path?(@base_path)
+    if absolute_path?(@base_path)
       loaded = load_once_path(@base_path)
       if loaded.nil?
         raise LoadError, error_message
@@ -17,7 +17,7 @@ class ScriptLoader
       end
     else
       $LOAD_PATH.each do |load_path|
-        unless absolete_path?(load_path)
+        unless absolute_path?(load_path)
           load_path = File.expand_path(load_path)
         end
         loaded = load_once_path(File.join(load_path, @base_path))
@@ -32,24 +32,24 @@ class ScriptLoader
     "cannot load such file -- #{@base_path}"
   end
 
-  def absolete_path?(path)
+  def absolute_path?(path)
     path.start_with?("/")
   end
 
   def load_once_path(path)
-    loaded = load_once_absolete_path(path)
+    loaded = load_once_absolute_path(path)
     return loaded unless loaded.nil?
 
     return nil unless File.extname(path).empty?
 
-    load_once_absolete_path("#{path}.rb")
+    load_once_absolute_path("#{path}.rb")
   end
 
-  def load_once_absolete_path(path)
+  def load_once_absolute_path(path)
     return false if $".include?(path)
     return false if @@loading_paths.key?(path)
 
-    return nil unless File.exist?(path)
+    return nil unless File.file?(path)
 
     @@loading_paths[path] = true
     load(path)
