@@ -14,13 +14,13 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include "groonga_in.h"
+#include "grn.h"
 #include <string.h>
 #include <limits.h>
-#include "pat.h"
-#include "output.h"
-#include "util.h"
-#include "normalizer_in.h"
+#include "grn_pat.h"
+#include "grn_output.h"
+#include "grn_util.h"
+#include "grn_normalizer.h"
 
 #define GRN_PAT_DELETED (GRN_ID_MAX + 1)
 
@@ -72,7 +72,7 @@ typedef struct {
     terminated == 1: key is terminated.
    */
   uint16_t bits;
-  /* length : 13, deleting : 1, immediate : 1 */
+  /* length: 13, immediate: 1, deleting: 1 */
 } pat_node;
 
 #define PAT_DELETING  (1<<1)
@@ -202,7 +202,7 @@ key_put(grn_ctx *ctx, grn_pat *pat, const uint8_t *key, int len)
     uint8_t *dest;
     KEY_AT(pat, res, dest, GRN_TABLE_ADD);
     if (!dest) { return 0; }
-    memcpy(dest, key, len);
+    grn_memcpy(dest, key, len);
   }
   pat->header->curr_key += len;
   return res;
@@ -227,7 +227,7 @@ pat_node_set_key(grn_ctx *ctx, grn_pat *pat, pat_node *n, const uint8_t *key, un
   PAT_LEN_SET(n, len);
   if (len <= sizeof(uint32_t)) {
     PAT_IMD_ON(n);
-    memcpy(&n->key, key, len);
+    grn_memcpy(&n->key, key, len);
   } else {
     PAT_IMD_OFF(n);
     n->key = key_put(ctx, pat, key, len);
@@ -238,8 +238,17 @@ pat_node_set_key(grn_ctx *ctx, grn_pat *pat, pat_node *n, const uint8_t *key, un
 /* delinfo operation */
 
 enum {
+  /* The delinfo is currently not used. */
   DL_EMPTY = 0,
+  /*
+   * stat->d refers to a deleting node (in a tree).
+   * The deletion requires an additional operation.
+   */
   DL_PHASE1,
+  /*
+   * stat->d refers to a deleted node (not in a tree).
+   * The node is pending for safety.
+   */
   DL_PHASE2
 };
 
@@ -265,32 +274,51 @@ delinfo_turn_2(grn_ctx *ctx, grn_pat *pat, grn_pat_delinfo *di)
   grn_id d, *p = NULL;
   pat_node *ln, *dn;
   // grn_log("delinfo_turn_2> di->d=%d di->ld=%d stat=%d", di->d, di->ld, di->stat);
-  if (di->stat != DL_PHASE1) { return GRN_SUCCESS; }
+  if (di->stat != DL_PHASE1) {
+    return GRN_SUCCESS;
+  }
   PAT_AT(pat, di->ld, ln);
-  if (!ln) { return GRN_INVALID_ARGUMENT; }
-  if (!(d = di->d)) { return GRN_INVALID_ARGUMENT; }
+  if (!ln) {
+    return GRN_INVALID_ARGUMENT;
+  }
+  d = di->d;
+  if (!d) {
+    return GRN_INVALID_ARGUMENT;
+  }
   PAT_AT(pat, d, dn);
-  if (!dn) { return GRN_INVALID_ARGUMENT; }
+  if (!dn) {
+    return GRN_INVALID_ARGUMENT;
+  }
   PAT_DEL_OFF(ln);
   PAT_DEL_OFF(dn);
   {
-    grn_id r, *p0;
+    grn_id *p0;
     pat_node *rn;
     int c0 = -1, c;
     uint32_t len = PAT_LEN(dn) * 16;
     const uint8_t *key = pat_node_get_key(ctx, pat, dn);
-    if (!key) { return GRN_INVALID_ARGUMENT; }
+    if (!key) {
+      return GRN_INVALID_ARGUMENT;
+    }
     PAT_AT(pat, 0, rn);
     p0 = &rn->lr[1];
-    while ((r = *p0)) {
+    for (;;) {
+      grn_id r = *p0;
+      if (!r) {
+        break;
+      }
       if (r == d) {
         p = p0;
         break;
       }
       PAT_AT(pat, r, rn);
-      if (!rn) { return GRN_FILE_CORRUPT; }
+      if (!rn) {
+        return GRN_FILE_CORRUPT;
+      }
       c = PAT_CHK(rn);
-      if (c <= c0 || len <= c) { break; }
+      if (c <= c0 || len <= c) {
+        break;
+      }
       if (c & 1) {
         p0 = (c + 1 < len) ? &rn->lr[1] : &rn->lr[0];
       } else {
@@ -610,8 +638,9 @@ _grn_pat_add(grn_ctx *ctx, grn_pat *pat, const uint8_t *key, uint32_t size, uint
   grn_id r, r0, *p0, *p1 = NULL;
   pat_node *rn, *rn0;
   int c, c0 = -1, c1 = -1, len;
-
   uint32_t cache_id = 0;
+
+  *new = 0;
   if (pat->cache) {
     const uint8_t *p = key;
     uint32_t length = size;
@@ -628,7 +657,6 @@ _grn_pat_add(grn_ctx *ctx, grn_pat *pat, const uint8_t *key, uint32_t size, uint
     }
   }
 
-  *new = 0;
   len = (int)size * 16;
   PAT_AT(pat, 0, rn0);
   p0 = &rn0->lr[1];
@@ -703,10 +731,10 @@ _grn_pat_add(grn_ctx *ctx, grn_pat *pat, const uint8_t *key, uint32_t size, uint
     if (*lkey && size2) {
       if (pat->header->garbages[0]) {
         r = pat->header->garbages[0];
-        pat->header->n_entries++;
-        pat->header->n_garbages--;
         PAT_AT(pat, r, rn);
         if (!rn) { return 0; }
+        pat->header->n_entries++;
+        pat->header->n_garbages--;
         pat->header->garbages[0] = rn->lr[0];
       } else {
         if (!(rn = pat_node_new(ctx, pat, &r))) { return 0; }
@@ -718,14 +746,14 @@ _grn_pat_add(grn_ctx *ctx, grn_pat *pat, const uint8_t *key, uint32_t size, uint
       if (pat->header->garbages[size2]) {
         uint8_t *keybuf;
         r = pat->header->garbages[size2];
-        pat->header->n_entries++;
-        pat->header->n_garbages--;
         PAT_AT(pat, r, rn);
         if (!rn) { return 0; }
-        pat->header->garbages[size2] = rn->lr[0];
         if (!(keybuf = pat_node_get_key(ctx, pat, rn))) { return 0; }
+        pat->header->n_entries++;
+        pat->header->n_garbages--;
+        pat->header->garbages[size2] = rn->lr[0];
         PAT_LEN_SET(rn, size);
-        memcpy(keybuf, key, size);
+        grn_memcpy(keybuf, key, size);
       } else {
         if (!(rn = pat_node_new(ctx, pat, &r))) { return 0; }
         pat_node_set_key(ctx, pat, rn, key, size);
@@ -1094,33 +1122,55 @@ _grn_pat_del(grn_ctx *ctx, grn_pat *pat, const char *key, uint32_t key_size, int
              grn_table_delete_optarg *optarg)
 {
   grn_pat_delinfo *di;
-  uint8_t direction;
   pat_node *rn, *rn0 = NULL, *rno;
-  int c, c0 = -1, ch;
+  int c = -1, c0 = -1, ch;
   uint32_t len = key_size * 16;
   grn_id r, otherside, *proot, *p, *p0 = NULL;
 
-  di = delinfo_new(ctx, pat); /* must be called before find rn */
+  /* delinfo_new() must be called before searching for rn. */
+  di = delinfo_new(ctx, pat);
   di->shared = shared;
+
+  /*
+   * Search a patricia tree for a given key.
+   * If the key exists, get its output node.
+   *
+   * rn, rn0: the output node and its previous node.
+   * rno: the other side of rn (the other destination of rn0).
+   * c, c0: checks of rn0 and its previous node.
+   * p, p0: pointers to transitions (IDs) that refer to rn and rn0.
+   */
   PAT_AT(pat, 0, rn);
-  c = -1;
   proot = p = &rn->lr[1];
   for (;;) {
-    if (!(r = *p)) { return GRN_INVALID_ARGUMENT; }
+    r = *p;
+    if (!r) {
+      return GRN_INVALID_ARGUMENT;
+    }
     PAT_AT(pat, r, rn);
-    if (!rn) { return GRN_FILE_CORRUPT; }
+    if (!rn) {
+      return GRN_FILE_CORRUPT;
+    }
     ch = PAT_CHK(rn);
-    if (len <= ch) { return GRN_INVALID_ARGUMENT; }
+    if (len <= ch) {
+      return GRN_INVALID_ARGUMENT;
+    }
     if (c >= ch) {
+      /* Output node found. */
       const uint8_t *k = pat_node_get_key(ctx, pat, rn);
-      if (!k) { return GRN_INVALID_ARGUMENT; }
-      if (key_size == PAT_LEN(rn) && !memcmp(k, key, key_size)) {
-        break; /* found */
-      } else {  return GRN_INVALID_ARGUMENT; }
+      if (!k) {
+        return GRN_INVALID_ARGUMENT;
+      }
+      if (key_size != PAT_LEN(rn) || memcmp(k, key, key_size)) {
+        return GRN_INVALID_ARGUMENT;
+      }
+      /* Given key found. */
+      break;
     }
     c0 = c;
     p0 = p;
-    if ((c = ch) & 1) {
+    c = ch;
+    if (c & 1) {
       p = (c + 1 < len) ? &rn->lr[1] : &rn->lr[0];
     } else {
       p = &rn->lr[nth_bit((uint8_t *)key, c, len)];
@@ -1131,26 +1181,54 @@ _grn_pat_del(grn_ctx *ctx, grn_pat *pat, const char *key, uint32_t key_size, int
       !optarg->func(ctx, (grn_obj *)pat, r, optarg->func_arg)) {
     return GRN_SUCCESS;
   }
-  direction = (rn0->lr[1] == r);
-  otherside = direction ? rn0->lr[0] : rn0->lr[1];
+  if (rn0->lr[0] == rn0->lr[1]) {
+    GRN_LOG(ctx, GRN_LOG_DEBUG, "*p0 (%d), rn0->lr[0] == rn0->lr[1] (%d)",
+            *p0, rn0->lr[0]);
+    return GRN_FILE_CORRUPT;
+  }
+  otherside = (rn0->lr[1] == r) ? rn0->lr[0] : rn0->lr[1];
+  if (otherside && r != otherside) {
+    PAT_AT(pat, otherside, rno);
+    if (!rno) {
+      return GRN_FILE_CORRUPT;
+    }
+  }
+
   if (rn == rn0) {
+    /* The last transition (p) is a self-loop. */
     di->stat = DL_PHASE2;
     di->d = r;
     if (otherside) {
-      PAT_AT(pat, otherside, rno);
-      if (rno && c0 < PAT_CHK(rno) && PAT_CHK(rno) <= c) {
+      if (c0 < PAT_CHK(rno) && PAT_CHK(rno) <= c) {
+        /* rno is an output node and will be a non-output node. */
         if (!delinfo_search(pat, otherside)) {
-          GRN_LOG(ctx, GRN_LOG_ERROR, "no delinfo found %d", otherside);
+          GRN_LOG(ctx, GRN_LOG_DEBUG, "no delinfo found %d", otherside);
         }
         PAT_CHK_SET(rno, 0);
       }
-      if (proot == p0 && !rno->check) { rno->lr[0] = rno->lr[1] = otherside; }
+      if (proot == p0 && !rno->check) {
+        const uint8_t *k = pat_node_get_key(ctx, pat, rno);
+        int direction = k ? (*k >> 7) : 1;
+        rno->lr[direction] = otherside;
+        rno->lr[!direction] = 0;
+      }
     }
     *p0 = otherside;
+  } else if ((!rn->lr[0] && rn->lr[1] == r) ||
+             (!rn->lr[1] && rn->lr[0] == r)) {
+    /* The output node has only a disabled self-loop. */
+    di->stat = DL_PHASE2;
+    di->d = r;
+    *p = 0;
   } else {
+    /* The last transition (p) is not a self-loop. */
     grn_pat_delinfo *ldi = NULL, *ddi = NULL;
-    if (PAT_DEL(rn)) { ldi = delinfo_search(pat, r); }
-    if (PAT_DEL(rn0)) { ddi = delinfo_search(pat, *p0); }
+    if (PAT_DEL(rn)) {
+      ldi = delinfo_search(pat, r);
+    }
+    if (PAT_DEL(rn0)) {
+      ddi = delinfo_search(pat, *p0);
+    }
     if (ldi) {
       PAT_DEL_OFF(rn);
       di->stat = DL_PHASE2;
@@ -1197,18 +1275,29 @@ _grn_pat_del(grn_ctx *ctx, grn_pat *pat, const char *key, uint32_t key_size, int
       }
     }
     if (*p0 == otherside) {
-      PAT_CHK_SET(rn0, 0);
-      if (proot == p0 && !rn0->check) { rn0->lr[0] = rn0->lr[1] = otherside; }
+      /* The previous node (*p0) has a self-loop (rn0 == rno). */
+      PAT_CHK_SET(rno, 0);
+      if (proot == p0) {
+        const uint8_t *k = pat_node_get_key(ctx, pat, rno);
+        int direction = k ? (*k >> 7) : 1;
+        rno->lr[direction] = otherside;
+        rno->lr[!direction] = 0;
+      }
     } else {
       if (otherside) {
-        PAT_AT(pat, otherside, rno);
-        if (rno && c0 < PAT_CHK(rno) && PAT_CHK(rno) <= c) {
+        if (c0 < PAT_CHK(rno) && PAT_CHK(rno) <= c) {
+          /* rno is an output node and will be a non-output node. */
           if (!delinfo_search(pat, otherside)) {
             GRN_LOG(ctx, GRN_LOG_ERROR, "no delinfo found %d", otherside);
           }
           PAT_CHK_SET(rno, 0);
         }
-        if (proot == p0 && !rno->check) { rno->lr[0] = rno->lr[1] = otherside; }
+        if (proot == p0 && !rno->check) {
+          const uint8_t *k = pat_node_get_key(ctx, pat, rno);
+          int direction = k ? (*k >> 7) : 1;
+          rno->lr[direction] = otherside;
+          rno->lr[!direction] = 0;
+        }
       }
       *p0 = otherside;
     }
@@ -1295,7 +1384,7 @@ grn_pat_get_key(grn_ctx *ctx, grn_pat *pat, grn_id id, void *keybuf, int bufsize
     if (KEY_NEEDS_CONVERT(pat, len)) {
       KEY_DEC(pat, keybuf, key, len);
     } else {
-      memcpy(keybuf, key, len);
+      grn_memcpy(keybuf, key, len);
     }
   }
   return len;
@@ -1341,9 +1430,9 @@ grn_pat_get_value(grn_ctx *ctx, grn_pat *pat, grn_id id, void *valuebuf)
     if (v) {
       if (valuebuf) {
         if (pat->obj.header.flags & GRN_OBJ_KEY_WITH_SIS) {
-          memcpy(valuebuf, v + sizeof(sis_node), value_size);
+          grn_memcpy(valuebuf, v + sizeof(sis_node), value_size);
         } else {
-          memcpy(valuebuf, v, value_size);
+          grn_memcpy(valuebuf, v, value_size);
         }
       }
       return value_size;
@@ -1377,7 +1466,7 @@ grn_pat_set_value(grn_ctx *ctx, grn_pat *pat, grn_id id,
         if (pat->obj.header.flags & GRN_OBJ_KEY_WITH_SIS) { v += sizeof(sis_node); }
         switch ((flags & GRN_OBJ_SET_MASK)) {
         case GRN_OBJ_SET :
-          memcpy(v, value, value_size);
+          grn_memcpy(v, value, value_size);
           return GRN_SUCCESS;
         case GRN_OBJ_INCR :
           switch (value_size) {
@@ -1671,55 +1760,52 @@ grn_pat_cursor_next(grn_ctx *ctx, grn_pat_cursor *c)
   while ((se = pop(c))) {
     grn_id id = se->id;
     int check = se->check, ch;
-    for (;;) {
-      if (id) {
-        PAT_AT(c->pat, id, node);
-        if (node) {
-          ch = PAT_CHK(node);
-          if (ch > check) {
+    while (id) {
+      PAT_AT(c->pat, id, node);
+      if (!node) {
+        break;
+      }
+      ch = PAT_CHK(node);
+      if (ch > check) {
+        if (c->obj.header.flags & GRN_CURSOR_DESCENDING) {
+          push(c, node->lr[0], ch);
+          id = node->lr[1];
+        } else {
+          push(c, node->lr[1], ch);
+          id = node->lr[0];
+        }
+        check = ch;
+        continue;
+      } else {
+        if (id == c->tail) {
+          c->sp = 0;
+        } else {
+          if (!c->curr_rec && c->tail) {
+            uint32_t lmin, lmax;
+            pat_node *nmin, *nmax;
+            const uint8_t *kmin, *kmax;
             if (c->obj.header.flags & GRN_CURSOR_DESCENDING) {
-              push(c, node->lr[0], ch);
-              id = node->lr[1];
+              PAT_AT(c->pat, c->tail, nmin);
+              PAT_AT(c->pat, id, nmax);
             } else {
-              push(c, node->lr[1], ch);
-              id = node->lr[0];
+              PAT_AT(c->pat, id, nmin);
+              PAT_AT(c->pat, c->tail, nmax);
             }
-            check = ch;
-            continue;
-          } else {
-            if (id == c->tail) {
+            lmin = PAT_LEN(nmin);
+            lmax = PAT_LEN(nmax);
+            kmin = pat_node_get_key(ctx, c->pat, nmin);
+            kmax = pat_node_get_key(ctx, c->pat, nmax);
+            if ((lmin < lmax) ?
+                (memcmp(kmin, kmax, lmin) > 0) :
+                (memcmp(kmin, kmax, lmax) >= 0)) {
               c->sp = 0;
-            } else {
-              if (!c->curr_rec && c->tail) {
-                uint32_t lmin, lmax;
-                pat_node *nmin, *nmax;
-                const uint8_t *kmin, *kmax;
-                if (c->obj.header.flags & GRN_CURSOR_DESCENDING) {
-                  PAT_AT(c->pat, c->tail, nmin);
-                  PAT_AT(c->pat, id, nmax);
-                } else {
-                  PAT_AT(c->pat, id, nmin);
-                  PAT_AT(c->pat, c->tail, nmax);
-                }
-                lmin = PAT_LEN(nmin);
-                lmax = PAT_LEN(nmax);
-                kmin = pat_node_get_key(ctx, c->pat, nmin);
-                kmax = pat_node_get_key(ctx, c->pat, nmax);
-                if ((lmin < lmax) ?
-                    (memcmp(kmin, kmax, lmin) > 0) :
-                    (memcmp(kmin, kmax, lmax) >= 0)) {
-                  c->sp = 0;
-                  break;
-                }
-              }
+              break;
             }
-            c->curr_rec = id;
-            c->rest--;
-            return id;
           }
         }
-      } else {
-        break;
+        c->curr_rec = id;
+        c->rest--;
+        return id;
       }
     }
   }
@@ -2773,7 +2859,7 @@ rk_emit(rk_tree_node *rn, char **str)
 #define RK_OUTPUT(e,l) do {\
   if (oc < oe) {\
     uint32_t l_ = (oc + (l) < oe) ? (l) : (oe - oc);\
-    memcpy(oc, (e), l_);\
+    grn_memcpy(oc, (e), l_);\
     oc += l_;\
     ic_ = ic;\
   }\
@@ -2863,7 +2949,7 @@ search_push(grn_ctx *ctx, grn_pat *pat, grn_pat_cursor *c,
           if (l + key_len <= GRN_TABLE_MAX_KEY_SIZE) {
             int ch = c0;
             grn_id i;
-            memcpy(key + key_len, e, l);
+            grn_memcpy(key + key_len, e, l);
             if ((i = sub_search(ctx, pat, id, &ch, key, key_len + l))) {
               search_push(ctx, pat, c, key, key_len + l, rn->next, i, ch, flags);
             }
